@@ -14,6 +14,15 @@ use warp::{Filter, Rejection, Reply};
 use warp_sessions::{
     self, CookieOptions, SameSiteCookieOption, Session, SessionStore, SessionWithStore,
 };
+use regex::Regex;
+use percent_encoding::percent_decode_str;
+use crate::routes::error::QueryParamValidationRejection;
+use serde::de::DeserializeOwned;
+
+
+pub trait Validate {
+    fn validate(&self) -> Result<(), Rejection> ;
+}
 
 #[derive(Deserialize, Serialize)]
 struct WithoutTokenQueryParams {
@@ -21,6 +30,14 @@ struct WithoutTokenQueryParams {
     edit: Option<bool>,
     data_type: Option<String>,
     relay_url: Option<String>,
+    js_message: Option<String>
+}
+
+impl Validate for WithoutTokenQueryParams {
+    fn validate(&self) -> Result<(), Rejection> {
+        validate_base64_query_param(self.js_message.clone())?;
+        Ok::<_, Rejection>(())
+    }
 }
 
 #[derive(Deserialize, Serialize)]
@@ -34,6 +51,14 @@ struct WithTokenQueryParams {
     edit: Option<bool>,
     data_type: Option<String>,
     relay_url: Option<String>,
+    js_message: Option<String>
+}
+
+impl Validate for WithTokenQueryParams {
+    fn validate(&self) -> Result<(), Rejection> {
+        validate_base64_query_param(self.js_message.clone())?;
+        Ok::<_, Rejection>(())
+    }
 }
 
 #[derive(Deserialize, Serialize, Debug)]
@@ -42,6 +67,15 @@ struct WithTokenPathParams {
     token: String,
 }
 
+pub fn validated_query_params<T: 'static + DeserializeOwned + Send + Validate>(
+) -> impl Filter<Extract = (T,), Error = Rejection> + Copy {
+    warp::query::<T>()
+        .and_then(move |param: T| async move {
+            param.validate()?;
+            Ok::<_, Rejection>(param)
+    })
+}
+
 pub fn without_token<S: SessionStore, R: Renderer, T: TokenGenerator>(
     session_store: S,
     render_engine: R,
@@ -49,7 +83,7 @@ pub fn without_token<S: SessionStore, R: Renderer, T: TokenGenerator>(
 ) -> impl Filter<Extract = (impl Reply,), Error = Rejection> + Clone {
     warp::any()
         .and(warp::path!("data" / String).map(|path| WithoutTokenPathParams { path }))
-        .and(warp::query::<WithoutTokenQueryParams>())
+        .and(validated_query_params::<WithoutTokenQueryParams>())
         .and(warp_sessions::request::with_session(
             session_store,
             Some(CookieOptions {
@@ -71,6 +105,7 @@ pub fn without_token<S: SessionStore, R: Renderer, T: TokenGenerator>(
                   session_with_store: SessionWithStore<S>,
                   token: String,
                   render_engine: R| async move {
+
                 let utv = UnsecureTemplateValues {
                     path: path_params.path.clone(),
                     token: token.clone(),
@@ -78,6 +113,7 @@ pub fn without_token<S: SessionStore, R: Renderer, T: TokenGenerator>(
                     edit: query_params.edit,
                     data_type: query_params.data_type,
                     relay_url: query_params.relay_url,
+                    js_message: query_params.js_message,
                 };
                 Ok::<_, Rejection>((
                     Rendered::new(
@@ -197,6 +233,7 @@ pub fn with_token<S: SessionStore, R: Renderer, T: TokenGenerator, H: Storer>(
                             css: query_params.css,
                             edit: query_params.edit,
                             relay_url: query_params.relay_url,
+                            js_message: query_params.js_message,
                         }),
                     },
                 )?;
@@ -255,6 +292,25 @@ pub fn with_token<S: SessionStore, R: Renderer, T: TokenGenerator, H: Storer>(
         .and_then(warp_sessions::reply::with_session)
 }
 
+fn validate_base64_query_param(str: Option<String>)-> Result<(), Rejection> {
+    match str {
+        Some(msg) => {
+            percent_decode_str(&msg)
+                .decode_utf8()
+                .map_err(|_| warp::reject::custom(QueryParamValidationRejection))
+                .and_then(|str| {
+                    let base64_regex = Regex::new(r"^[A-Za-z0-9+/]+={0,2}$").unwrap();
+                    match base64_regex.is_match(&str.to_string()) {
+                        true => Ok::<_, Rejection>(()),
+                        false => Err(warp::reject::custom(QueryParamValidationRejection))
+                    }
+                })
+
+        },
+        None => Ok(())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     mod with_token {
@@ -357,6 +413,7 @@ mod tests {
                         css: None,
                         edit: None,
                         relay_url: None,
+                        js_message: None,
                     });
                     template.value == expected_value
                 })
@@ -482,6 +539,7 @@ mod tests {
         use crate::token::tests::MockTokenGenerator;
         use std::sync::Arc;
         use warp_sessions::MemoryStore;
+        use mockall::PredicateStrExt;
 
         #[tokio::test]
         async fn without_token_with_no_query_params() {
@@ -498,6 +556,7 @@ mod tests {
                         edit: None,
                         data_type: None,
                         relay_url: None,
+                        js_message: None
                     });
 
                     template.value == expected_value
@@ -544,6 +603,7 @@ mod tests {
                         edit: None,
                         data_type: None,
                         relay_url: None,
+                        js_message: None
                     });
 
                     template.value == expected_value
@@ -590,6 +650,7 @@ mod tests {
                         edit: Some(true),
                         data_type: None,
                         relay_url: None,
+                        js_message: None
                     });
                     template.value == expected_value
                 })
@@ -635,6 +696,7 @@ mod tests {
                         edit: Some(false),
                         data_type: None,
                         relay_url: None,
+                        js_message: None
                     });
                     template.value == expected_value
                 })
@@ -664,5 +726,82 @@ mod tests {
                 .await;
             assert_eq!(res.status(), 200);
         }
+
+
+        #[tokio::test]
+        async fn test_without_token_with_js_message_valid() {
+            let js_message = "dXBkYXRl";
+
+            let session_store = MemoryStore::new();
+            let mut render_engine = MockRenderer::new();
+            render_engine
+                .expect_render()
+                .withf(move |template: &RenderTemplate| {
+                    let expected_value = TemplateValues::Unsecure(UnsecureTemplateValues {
+                        path: ".testKey.".to_owned(),
+                        token: "E0AE2C1C9AA2DB85DFA2FF6B4AAC7A5E51FFDAA3948BECEC353561D513E59A9C"
+                            .to_owned(),
+                        css: None,
+                        edit: Some(false),
+                        data_type: None,
+                        relay_url: None,
+                        js_message: Some(js_message.to_owned())
+                    });
+                    template.value == expected_value
+                })
+                .times(1)
+                .return_once(move |_| Ok("".to_string()));
+
+            let mut token_generator = MockTokenGenerator::new();
+            token_generator
+                .expect_generate_token()
+                .times(1)
+                .returning(|| {
+                    Ok(
+                        "E0AE2C1C9AA2DB85DFA2FF6B4AAC7A5E51FFDAA3948BECEC353561D513E59A9C"
+                            .to_owned(),
+                    )
+                });
+
+            let without_token_filter = get::without_token(
+                session_store,
+                Arc::new(render_engine),
+                Arc::new(token_generator),
+            );
+
+            let res = warp::test::request()
+                .path(&format!("/data/.testKey.?edit=false&js_message={}", js_message))
+                .reply(&without_token_filter)
+                .await;
+            assert_eq!(res.status(), 200);
+        }
+
+        #[tokio::test]
+        async fn test_without_token_with_js_message_invalid() {
+            let js_message = "invalid%5C%22%29%3B+mesaage%7D%7B";
+
+            let session_store = MemoryStore::new();
+            let mut render_engine = MockRenderer::new();
+            render_engine
+                .expect_render()
+                .times(0);
+
+            let mut token_generator = MockTokenGenerator::new();
+            token_generator
+                .expect_generate_token()
+                .times(0);
+
+            let without_token_filter = get::without_token(
+                session_store,
+                Arc::new(render_engine),
+                Arc::new(token_generator),
+            );
+
+            let res = warp::test::request()
+                .path(&format!("/data/.testKey.?edit=false&js_message={}", js_message))
+                .reply(&without_token_filter)
+                .await;
+            assert_eq!(res.status(), 500);
+        }
     }
 }
